# -*- coding: utf-8 -*-
"""Onto-crawl API section."""

import re
from os.path import join, splitext
from pathlib import Path
from typing import Generator, Optional

import kgcl_schema.grammar.parser as kgcl_parser
from github import Github
from github.Issue import Issue
from oaklib.cli import query_terms_iterator
from oaklib.interfaces.patcher_interface import PatcherInterface
from oaklib.selector import get_resource_from_shorthand

HOME_DIR = Path(__file__).resolve().parents[2]
SRC = HOME_DIR / "src/onto_crawler"
TESTS = HOME_DIR / "tests"
ONTOLOGY_RESOURCE = TESTS / "resources/fbbt.obo"
RESOURCE_DICT = {"hrshdhgd/mondo": "src/ontology/mondo-edit.obo"}

# Token.txt unique to every user.
# For more information:
#   https://docs.github.com/en/enterprise-server@3.4/authentication/keeping-your-account-and-data-secure/creating-a-personal-access-token
# Save the token in a txt file as named below.
SRC = Path(__file__).parent
TOKEN_FILE = join(SRC, "token.txt")

with open(TOKEN_FILE, "r") as t:
    TOKEN = t.read().rstrip()

g = Github(TOKEN)
# Example for API: https://pygithub.readthedocs.io/en/latest/examples.html

RAW_DATA = "_rawData"
ISSUE_KEYS = [
    # 'repository_url',
    # 'html_url',
    "number",
    "title",
    # 'user',
    "labels",
    # 'assignee',
    # 'assignees',
    # "comments",
    # "created_at",
    # "updated_at",
    "body",
]


def get_issues(
    repository_name: str,
    title_search: Optional[str] = None,
    label: Optional[str] = None,
    number: Optional[int] = 0,
    state: str = "open",
) -> Generator:
    """Get issues of specific states from a Github repository.

    :param repository_name: Name of the repository [org/repo]
    :param title_search: Regex for title of the issue.
    :param state: State of the issue e.g. open, close etc., defaults to "open"
    :yield: Issue names that match the regex/label/number.
    """
    repo = g.get_repo(repository_name)
    label_object = None
    if label:
        label_object = repo.get_label(label)

    issues = repo.get_issues(state=state)

    for issue in issues:
        if title_search is None and label_object is None and number == 0:
            yield issue
        else:
            if title_search and re.match(title_search, issue.title):
                yield _extract_info_from_issue_object(issue)
            if label_object and label_object in issue.labels:
                yield _extract_info_from_issue_object(issue)
            if number and number == issue.number:
                yield _extract_info_from_issue_object(issue)


def _extract_info_from_issue_object(issue: Issue) -> dict:
    issue_as_dict = issue.__dict__
    important_info = {k: issue_as_dict[RAW_DATA][k] for k in ISSUE_KEYS}
    important_info["body"] = _make_sense_of_body(important_info["body"])
    return important_info


def _make_sense_of_body(body: str) -> list:
    return body.replace("\r", "").replace("\n", "").split("* ")[1:]


def get_all_labels_from_repo(repository_name: str) -> dict:
    """Get all labels available in a repository for tagging issues on creation.

    :param repository_name: Name of the repository.
    :return: A dictionary of {name: description}
    """
    repo = g.get_repo(repository_name)
    return {label.name: label.description for label in repo.get_labels()}


def process_issue_via_kgcl(
    repository_name: str, body: list, output: str = None
):
    """Pass KGCL commands in the body to OAK.

    :param repository_name: Name of the repository.
    :param body: A list of commands.
    :param output: Path to where the output is written, defaults to None
    """
    resource = get_resource_from_shorthand(str(ONTOLOGY_RESOURCE))
    impl_class = resource.implementation_class
    impl_obj: PatcherInterface = impl_class(resource)

    if output:
        _, ext = splitext(str(output))
    else:
        fn, ext = splitext(str(resource.local_path))
        output = fn + "_new" + ext

    output_format = ext.replace(".", "")

    for command in body:
        change = kgcl_parser.parse_statement(command)
        # TODO: There must be a better way to identify label in command.
        if ":" not in command:
            change.about_node = list(
                query_terms_iterator([change.about_node], impl_obj)
            )[0]

        impl_obj.apply_patch(change)

    impl_obj.dump(output, output_format)
    # Update content of file
    # repo = g.get_repo(repository_name)
    # content = repo.get_contents(path=RESOURCE_DICT[repository_name])
    # repo.update_file(content.path,"test",content.sha, branch="test")

    # Create PR
    # body = '''
    # SUMMARY
    # Add synonyms to ontology
    # '''
    # pr = repo.create_pull(
    #       title="Add synonyms to ontology",
    #       body=body, head="develop",
    #       base="master"
    # )



# -*- coding: utf-8 -*-

"""Command line interface for :mod:`onto_crawler`."""

import logging
from typing import TextIO

import click

from onto_crawler import __version__
from onto_crawler.api import (
    get_all_labels_from_repo,
    get_issues,
    process_issue_via_kgcl,
)

__all__ = [
    "main",
]

logger = logging.getLogger(__name__)

BODY = "body"


@click.group()
@click.option("-v", "--verbose", count=True)
@click.option("-q", "--quiet")
@click.version_option(__version__)
def main(verbose: int, quiet: bool):
    """CLI for onto_crawler.

    :param verbose: Verbosity while running.
    :param quiet: Boolean to be quiet or verbose.
    """
    if verbose >= 2:
        logger.setLevel(level=logging.DEBUG)
    elif verbose == 1:
        logger.setLevel(level=logging.INFO)
    else:
        logger.setLevel(level=logging.WARNING)
    if quiet:
        logger.setLevel(level=logging.ERROR)


# All frequently used options.
repo_option = click.option(
    "-r",
    "--repo",
    default="hrshdhgd/mondo",
    help="Org/name of the github repo.",
)

issue_number_option = click.option(
    "-n",
    "--number",
    type=int,
    help="Filter based on issue number.",
)

label_option = click.option(
    "-l",
    "--label",
    help="Filter based on a search for label of issue.",
)

state_option = click.option(
    "-s",
    "--state",
    default="open",
    help="State of the issue. [open, close etc.]",
)

output_option = click.option(
    "-o",
    "--output",
    help="Output could be a file or sys.stdout.",
)


@main.command()
@repo_option
@state_option
@output_option
@click.option(
    "-t",
    "--title-search",
    help="Filter based on a search for pattern within title of issue.",
)
@label_option
@issue_number_option
def issues(
    repo: str,
    state: str,
    title_search: str,
    label: str,
    number: int,
    output: TextIO,
):
    """Get issues based on different options.

    :param repo: GitHub repository name [org/repo_name]
    :param state: State of issue ["open", "close" etc.]
    :param title_search: String lookup in title of issue.
    :param label: Label lookup for issues.
    :param output: Output location.
    """
    for issue in get_issues(
        repository_name=repo,
        state=state,
        title_search=title_search,
        label=label,
        number=number,
    ):
        print(issue, file=output)


@main.command("get-labels")
@repo_option
def get_labels(repo: str):
    """Get all labels for issues.

    :param repo: GitHub repository name [org/repo_name]
    """
    print(get_all_labels_from_repo(repo))


@main.command()
@repo_option
@label_option
@issue_number_option
@state_option
@output_option
def process_issue(repo: str, label: str, number: int, state: str, output: str):
    """Run processes based on issue label.

    :param repo: GitHub repository name [org/repo_name]
    :param label: Label of issues.
    :param state: State of issue ["open", "close" etc.]
    """
    for issue in get_issues(
        repository_name=repo, label=label, number=number, state=state
    ):
        if output:
            new_output = str(issue["number"]) + "_" + output
        else:
            new_output = output
        process_issue_via_kgcl(
            repository_name=repo, body=issue[BODY], output=new_output
        )
        # Open a new PR corresponding to the issue


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

"""Crawl github for ontology related issues."""

import sys

from .api import *  # noqa

if sys.version_info >= (3, 8):
    import importlib.metadata

    __version__ = importlib.metadata.version(__name__)
elif sys.version_info < (3, 8):
    import importlib_metadata

    __version__ = importlib_metadata.version(__name__)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['onto_crawler']

package_data = \
{'': ['*']}

install_requires = \
['PyGithub>=1.55,<2.0', 'oaklib>=0.1.31,<0.2.0']

entry_points = \
{'console_scripts': ['ocrawl = onto_crawler.cli:main']}

setup_kwargs = {
    'name': 'onto-crawler',
    'version': '0.1.1',
    'description': 'Crawl github for ontology related issues.',
    'long_description': '<!--\n<p align="center">\n  <img src="https://github.com/hrshdhgd/onto-crawler/raw/main/docs/source/logo.png" height="150">\n</p>\n-->\n\nCrawl github for ontology specific issues.\n\n## Getting Started\n\n[Read the docs](https://hrshdhgd.github.io/onto-crawler/index.html)\n\n<!-- ## Installation -->\n\n<!-- Uncomment this section after first release\nThe most recent release can be installed from\n[PyPI](https://pypi.org/project/onto_crawler/) with:\n\n```bash\n$ pip install onto-crawler\n```\n-->\n\n<!-- The most recent code and data can be installed directly from GitHub with:\n\n```bash\n$ pip install git+https://github.com/hrshdhgd/onto-crawler.git\n``` -->\n\n<!-- ## Contributing\n\nContributions, whether filing an issue, making a pull request, or forking, are appreciated. See\n[CONTRIBUTING.md](https://github.com/hrshdhgd/onto-crawler/blob/master/.github/CONTRIBUTING.md) for more information on getting involved. -->\n\n\n\n### License\n\nThe code in this package is licensed under the MIT License.\n',
    'author': 'Harshad Hegde',
    'author_email': 'hhegde@lbl.gov',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-

"""Version information for :mod:`onto_crawler`.

Run with ``python -m onto_crawler.version``
"""

import os
from subprocess import CalledProcessError, check_output  # noqa: S404

__all__ = [
    "VERSION",
    "get_version",
    "get_git_hash",
]

VERSION = "0.0.1-dev"


def get_git_hash() -> str:
    """Get the :mod:`onto_crawler` git hash.

    :return: git hash.
    """
    with open(os.devnull, "w") as devnull:
        try:
            ret = check_output(  # noqa: S603,S607
                ["git", "rev-parse", "HEAD"],
                cwd=os.path.dirname(__file__),
                stderr=devnull,
            )
        except CalledProcessError:
            return "UNHASHED"
        else:
            return ret.strip().decode("utf-8")[:8]


def get_version(with_git_hash: bool = False):
    """Get the :mod:`onto_crawler` version string, including a git hash.

    :param with_git_hash: Get version with git hash.    , defaults to False
    :return: None
    """
    return f"{VERSION}-{get_git_hash()}" if with_git_hash else VERSION


if __name__ == "__main__":
    print(get_version(with_git_hash=True))  # noqa:T201
